@@ -1,6 +1,6 @@
 
 /*
- * $Id: cimXmlParser.c,v 1.33 2009/06/05 20:09:34 buccella Exp $
+ * $Id: cimXmlParser.c c29c15e on Wed Feb 19 18:47:31 2014 -0500 by Dave Heller $
  *
  *  © Copyright IBM Corp. 2005, 2007
  *
@@ -1696,8 +1696,12 @@ yylex(YYSTYPE * lvalp, ParserControl * parm)
   _SFCB_RETURN(0);
 }
 
+/*
+ * The bison %parse-param directive modifies the yyerror() signature as well as
+ * yyparse(), so we must handle the extra parm even though we don't use it.
+ */
 int
-yyerror(char *s)
+yyerror(void __attribute__ ((unused)) *parm, char *s)
 {
   printf("*-* yyerror: %s\n", s);
   return 5;
