@@ -229,10 +229,15 @@ extern int fallback_ipv4;
 #define TERMINATE(x) { if (!doFork) _SFCB_RETURN(x); commClose(conn_fd); exit(x); }
 
 void
-initHttpProcCtl(int p)
+initHttpProcCtl(int p, int adapterNum)
 {
-  httpProcSemKey = ftok(SFCB_BINARY, 'H');
-  httpWorkSemKey = ftok(SFCB_BINARY, 'W');
+  /*
+   * ftok uses only the lowest 8 bits of proj_id, for 256 possible keys per
+   * pathname.  We need two keys per pathname per process, so we can support
+   * up to 128 adapters before colliding.
+   * */
+  httpProcSemKey = ftok(SFCB_BINARY, adapterNum + 0);
+  httpWorkSemKey = ftok(SFCB_BINARY, adapterNum + 127);
   union semun     sun;
   int             i;
 
@@ -264,7 +269,7 @@ initHttpProcCtl(int p)
        semget(httpWorkSemKey, 1, IPC_CREAT | IPC_EXCL | 0600)) == -1) {
     char           *emsg = strerror(errno);
     mlogf(M_ERROR, M_SHOW,
-          "\n--- Http ProcWork semaphore create key: 0x%x failed: %s\n",
+          "\n--- Http Work semaphore create key: 0x%x failed: %s\n",
           httpWorkSemKey, emsg);
     mlogf(M_ERROR, M_SHOW,
           "     use \"ipcrm -S 0x%x\" to remove semaphore\n\n",
@@ -1444,7 +1449,6 @@ handleHttpRequest(int connFd, int __attribute__ ((unused)) sslMode)
   CommHndl        conn_fd;
   int             isReady;
   fd_set          httpfds;
-  //  struct sembuf   procReleaseUnDo = { 0, 1, SEM_UNDO };
   struct timeval  httpTimeout;
   int             breakloop;
 
@@ -1454,10 +1458,16 @@ handleHttpRequest(int connFd, int __attribute__ ((unused)) sslMode)
     _SFCB_TRACE(1, ("--- Forking request processor"));
     semAcquire(httpWorkSem, 0);
     semAcquire(httpProcSem, 0);
-    for (httpProcIdX = 0; httpProcIdX < hMax; httpProcIdX++)
-      if (semGetValue(httpProcSem, httpProcIdX + 1) == 0)
+    for (httpProcIdX = 1; httpProcIdX <= hMax; httpProcIdX++)
+      if (semGetValue(httpProcSem, httpProcIdX) == 0)
         break;
-    //    procReleaseUnDo.sem_num = httpProcIdX + 1;
+
+    /* This should not happen, but if it does exit the req handler gracefully */
+    if (httpProcIdX > hMax) {
+      semRelease(httpProcSem, 0);
+      semRelease(httpWorkSem, 0);
+      return; 
+    }
 
     sessionId++;
     r = fork();
@@ -1470,11 +1480,21 @@ handleHttpRequest(int connFd, int __attribute__ ((unused)) sslMode)
       processName = "CIMREQ-Processor";
       semRelease(httpProcSem, 0);
       semAcquireUnDo(httpProcSem, 0);
-      semReleaseUnDo(httpProcSem, httpProcIdX + 1);
+      semReleaseUnDo(httpProcSem, httpProcIdX);
       semRelease(httpWorkSem, 0);
       atexit(releaseAuthHandle);
       atexit(uninitGarbageCollector);
       atexit(sunsetControl);
+
+      /* Label the process by modifying the cmdline */
+      extern void append2Argv(char *appendstr);
+      extern unsigned int labelProcs;
+      if (labelProcs) {
+        append2Argv(" -reqhandler: ");
+        char handlerId[8];
+        sprintf(handlerId, "%d", httpProcIdX);
+        append2Argv(handlerId);
+      }
     }
     /*
      * parent's thread of execution 
@@ -2015,8 +2035,10 @@ initSSL()
                  *fnt,
                  *fnl,
                  *fcert,
+                 *fdhp,
                  *sslCiphers;
-  int             rc;
+  int             rc,
+                  escsp;
 
   if (ctx)
     SSL_CTX_free(ctx);
@@ -2062,10 +2084,17 @@ initSSL()
   }
 
   /*
-   * SSLv2 is pretty old; no one should be needing it any more 
+   * Set options
    */
-  SSL_CTX_set_options(ctx, SSL_OP_ALL | SSL_OP_NO_SSLv2 |
-                      SSL_OP_SINGLE_DH_USE);
+  SSL_CTX_set_options(ctx, SSL_OP_ALL | 
+                           SSL_OP_NO_SSLv2 |
+                           SSL_OP_SINGLE_DH_USE);
+
+  if (!getControlBool("enableSslCipherServerPref", &escsp) && escsp) {
+    _SFCB_TRACE(1, ("---  enableSslCipherServerPref = true"));
+    SSL_CTX_set_options(ctx, SSL_OP_CIPHER_SERVER_PREFERENCE);
+  }
+
   /*
    * Set valid ciphers
    */
@@ -2074,12 +2103,57 @@ initSSL()
   if (SSL_CTX_set_cipher_list(ctx, sslCiphers) != 1)
     intSSLerror("Error setting cipher list (no valid ciphers)");
 
+#if (defined HEADER_DH_H && !defined OPENSSL_NO_DH)
+  /*
+   * Set DH parameters file for ephemeral key generation
+   */
+  getControlChars("sslDhParamsFilePath", &fdhp);
+  if (fdhp) {
+    _SFCB_TRACE(1, ("---  sslDhParamsFilePath = %s", fdhp));
+    BIO *dhpbio = BIO_new_file(fdhp, "r");
+    DH *dh_tmp = PEM_read_bio_DHparams(dhpbio, NULL, NULL, NULL);
+    BIO_free(dhpbio);
+    if (dh_tmp) {
+      SSL_CTX_set_tmp_dh(ctx, dh_tmp);
+      DH_free(dh_tmp);
+    } else {
+      unsigned long sslqerr = ERR_get_error();
+      mlogf(M_ERROR,M_SHOW,"--- Failure reading DH params file: %s (%s)\n",
+          fdhp, sslqerr != 0 ? ERR_error_string(sslqerr, NULL) :
+              "unknown openssl error");
+      intSSLerror("Error setting DH params for SSL");
+    }
+  }
+#endif                          // HEADER_DH_H
+
+#if (defined HEADER_EC_H && !defined OPENSSL_NO_EC)
+  /*
+   * Set ECDH curve name for ephemeral key generation
+   */
+  char *ecdh_curve_name;
+  getControlChars("sslEcDhCurveName", &ecdh_curve_name);
+  if (ecdh_curve_name) {
+    _SFCB_TRACE(1, ("---  sslEcDhCurveName = %s", ecdh_curve_name));
+    EC_KEY *ecdh = EC_KEY_new_by_curve_name(OBJ_sn2nid(ecdh_curve_name));
+    if (ecdh) {
+      SSL_CTX_set_tmp_ecdh(ctx, ecdh);
+      EC_KEY_free(ecdh);
+    } else {
+      unsigned long sslqerr = ERR_get_error();
+      mlogf(M_ERROR, M_SHOW, "--- Failure setting ECDH curve name (%s): %s\n",
+          ecdh_curve_name, sslqerr != 0 ?
+              ERR_error_string(sslqerr, NULL ) : "unknown openssl error");
+      intSSLerror("Error setting ECDH curve name for SSL");
+    }
+  }
+#endif                          // HEADER_EC_H
+
   sslReloadRequested = 0;
 }
 #endif                          // USE_SSL
 
 int
-httpDaemon(int argc, char *argv[], int sslMode, char *ipAddr,
+httpDaemon(int argc, char *argv[], int sslMode, int adapterNum, char *ipAddr,
     sa_family_t ipAddrFam, int sfcbPid)
 {
 
@@ -2159,7 +2233,7 @@ httpDaemon(int argc, char *argv[], int sslMode, char *ipAddr,
     doFork = 0;
   }
 
-  initHttpProcCtl(hMax);
+  initHttpProcCtl(hMax, adapterNum);
 
   if (getControlBool("doBasicAuth", &doBa))
     doBa = 0;
@@ -2236,6 +2310,26 @@ httpDaemon(int argc, char *argv[], int sslMode, char *ipAddr,
           keepaliveMaxRequest);
   }
 
+  /* Label the process by modifying the cmdline */
+  extern void append2Argv(char *appendstr);
+  extern unsigned int labelProcs;
+  if (labelProcs) {
+    append2Argv(NULL);
+    append2Argv("-proc:HttpDaemon ");
+    append2Argv("-ip:");
+    char *l = "";
+    char *r = "";
+#ifdef HAVE_IPV6
+    if (ipAddrFam == AF_INET6) {
+      l = "[";
+      r = "]";
+    }
+#endif
+    append2Argv(l);
+    append2Argv(ipAddr);
+    append2Argv(r);
+  }
+
   if (enableHttp) {
     httpListenFd = getSocket(ipAddrFam);
   }
