@@ -62,7 +62,7 @@ extern int      init_sfcBroker();
 extern CMPIBroker *Broker;
 extern void     initProvProcCtl(int);
 extern void     processTerminated(int pid);
-extern int      httpDaemon(int argc, char *argv[], int sslMode, char *ipAddr, sa_family_t ipAddrFam);
+extern int      httpDaemon(int argc, char *argv[], int sslMode, int adapterNum, char *ipAddr, sa_family_t ipAddrFam);
 extern void     processProviderMgrRequests();
 
 extern int      stopNextProc();
@@ -104,6 +104,12 @@ extern char    *configfile;
 extern char    *ip4List;
 extern char    *ip6List;
 
+extern char   **origArgv;
+extern int      origArgc;
+extern unsigned int labelProcs;
+extern void     append2Argv(char *appendstr);
+extern void     restoreOrigArgv(int removePad);
+
 #ifdef HAVE_IPV6
 int fallback_ipv4 = 1;
 #endif
@@ -187,6 +193,9 @@ static pthread_cond_t sdCnd = PTHREAD_COND_INITIALIZER;
 static int      stopping = 0;
 extern int      remSem();
 
+/* secs to wait for a process to die during shutdown [sfcb#94] */
+static int      sigChldWaitTime = 3;
+
 static void
 stopBroker(void *p)
 {
@@ -226,7 +235,7 @@ stopBroker(void *p)
 
     if (adaptersStopped == 0) {
       pthread_mutex_lock(&sdMtx);
-      waitTime.tv_sec = time(NULL) + 1; //5
+      waitTime.tv_sec = time(NULL) + sigChldWaitTime;
       waitTime.tv_nsec = 0;
       if (sa == 0)
         fprintf(stderr, "--- Stopping adapters\n");
@@ -245,7 +254,7 @@ stopBroker(void *p)
 
     if (adaptersStopped) {
       pthread_mutex_lock(&sdMtx);
-      waitTime.tv_sec = time(NULL) + 1; //5
+      waitTime.tv_sec = time(NULL) + sigChldWaitTime;
       waitTime.tv_nsec = 0;
       if (sp == 0)
         fprintf(stderr, "--- Stopping providers\n");
@@ -272,7 +281,7 @@ stopBroker(void *p)
   closeLogging(1);
   free((void *)sfcBrokerStart);
 
-  pthread_mutex_unlock(&syncMtx);
+//  pthread_mutex_unlock(&syncMtx); /* [sfcb#95] */
 
   unloadHostnameLib();
 
@@ -338,6 +347,8 @@ handleSigHup(int sig)
 
   if (sfcBrokerPid == currentProc) {
     restartBroker = 1;
+    if (labelProcs)
+      restoreOrigArgv(1);
     fprintf(stderr, "--- Restarting %s\n", processName);
     pthread_attr_init(&tattr);
     pthread_attr_setdetachstate(&tattr, PTHREAD_CREATE_DETACHED);
@@ -397,7 +408,17 @@ handleSigChld(int __attribute__ ((unused)) sig)
 static int 
 reStartHttpd(void)
 {
-   return startHttpd(restartArgc, restartArgv, sslMode);
+   /* Restore orig argv or startup banner will be hosed on httpd restart */
+   if (labelProcs)
+     restoreOrigArgv(0);
+
+   int rc = startHttpd(restartArgc, restartArgv, sslMode);
+  
+   /* Relabel the process */
+   if (labelProcs)
+     append2Argv(NULL);
+
+   return rc;
 }
 
 static void
@@ -418,7 +439,7 @@ handleSigUsr2(int __attribute__ ((unused)) sig)
    }
    while(!adaptersStopped) {
        pthread_mutex_lock(&sdMtx);
-       waitTime.tv_sec=time(NULL)+1; //5
+       waitTime.tv_sec=time(NULL) + sigChldWaitTime;
        waitTime.tv_nsec=0;
        if (sa==0) fprintf(stderr,"--- Stopping http adapters\n");
        sa++;
@@ -514,10 +535,10 @@ startHttpd(int argc, char *argv[], int sslMode)
           exit(2);
         }
       }
-      if (httpDaemon(argc, argv, sslMode, (ipAddrList + i)->addrStr,
+      if (httpDaemon(argc, argv, sslMode, i+1, (ipAddrList + i)->addrStr,
           (ipAddrList + i)->addrFam)) {
         //kill(sfcPid, 3);          /* if port in use, shutdown */
-                                    /* (don't do this anymore - xxxxxxx) */
+                                    /* (don't do this anymore - 3597806) */
       }
       closeSocket(&sfcbSockets,cRcv,"startHttpd");
       closeSocket(&resultSockets,cAll,"startHttpd");
@@ -637,6 +658,10 @@ main(int argc, char *argv[])
   char           *pauseStr;
   int             daemonize=0;
 
+  /* Note we will see no mlogf() output prior to this flag being set. If the
+   * flag is set but startLogging() has not yet been called, mlogf() will write
+   * to syslog directly. After startLogging(), mlogf() will write through the
+   * the logging facility. */
   sfcbUseSyslog=1;
 
   name = strrchr(argv[0], '/');
@@ -651,8 +676,8 @@ main(int argc, char *argv[])
   provPauseStr = getenv("SFCB_PAUSE_PROVIDER");
   httpPauseStr = getenv("SFCB_PAUSE_CODEC");
   currentProc = sfcBrokerPid = getpid();
-  restartArgc = argc;
-  restartArgv = argv;
+  restartArgc = origArgc = argc;
+  restartArgv = origArgv = argv;
 
   exFlags = 0;
 
@@ -748,7 +773,7 @@ main(int argc, char *argv[])
     }
   }
 
-  if (optind < argc) {
+  if (optind < argc && *argv[optind] != 'X') { /* Ignore the pad argument */
     fprintf(stderr, "SFCB not started: unrecognized config property %s\n",
             argv[optind]);
     usage(1);
@@ -766,6 +791,41 @@ main(int argc, char *argv[])
       currentProc=sfcBrokerPid=getpid(); /* req. on some systems */
   }
 
+  char *envLabelProcs = getenv("SFCB_LABELPROCS");
+  if (envLabelProcs && *envLabelProcs) {
+    char *endptr;
+    long val = strtol(envLabelProcs, &endptr, 10);
+    if ((*endptr =='\0') && (val > 0) && (val < 1024))
+      labelProcs = (unsigned int) val;
+  }
+
+  if (labelProcs) {
+    if (*argv[argc-1] != 'X') {
+      /* Create an expanded argv */
+      char **newArgv = malloc(sizeof(char*) * (argc + 2));
+      memcpy(newArgv, argv, sizeof(char*) * argc);
+
+      /* Create a pad argument of appropriate length */
+      char *padArg = malloc(labelProcs + 1);
+      for (i=0; i < (int)labelProcs; i++)
+        padArg[i] = 'X';
+      padArg[i] = '\0';
+
+      /* Add pad argument and null terminator */
+      newArgv[argc] = padArg;
+      newArgv[argc + 1] = NULL;
+
+      /* Restart with expanded argv */
+      fprintf(stderr,
+          "--- %s V" sfcHttpDaemonVersion " performing labelProcs allocation - %d\n",
+          name, currentProc);
+      execvp(newArgv[0], newArgv);
+      fprintf(stderr, "--- failed to execv for labelProcs allocation: %s\n",
+          strerror(errno));
+      exit(1);
+    }
+  }
+
   startLogging(syslogLevel,1);
 
   mlogf(M_NOTICE, M_SHOW, "--- %s V" sfcHttpDaemonVersion " started - %d\n",
@@ -897,6 +957,15 @@ main(int argc, char *argv[])
     trimws = 0;
   }
 
+  char *eoe;
+  extern int useCDATA;
+  if (!getControlChars("embeddedObjEncoding",&eoe) && !strcasecmp(eoe,"CDATA")) {
+    useCDATA = 1;
+    mlogf(M_INFO,M_SHOW,"--- Use CDATA for EmbeddedObject encoding\n");
+  } else {
+    mlogf(M_INFO,M_SHOW,"--- Use XML escaping for EmbeddedObject encoding\n");
+  }
+    
   if ((enableHttp || enableHttps) && dSockets > 0) {
     startHttp = 1;
   }
@@ -1006,6 +1075,11 @@ mlogf(M_INFO, M_SHOW, "--- Request handlers enabled:%s\n",rtmsg);
   setSignal(SIGCHLD, handleSigChld, 0);
   setSignal(SIGUSR2, handleSigUsr2, 0);
 
+  /* Label the process by modifying the cmdline */
+  if (labelProcs) {
+    append2Argv("-proc:Main");
+  }
+
   processProviderMgrRequests();
 
   stopBroker(NULL);
